/*
 Tracks calls to action.

 Usage:

 Include the tracking.js file by adding the below tracking code right before the
 closing body element (i.e. </body> element).

 Replace [CAMPAIGNCHAIN INSTALLATION] with the URL of the root of your CampaignChain
 installation, e.g. http://www.example.com/bundles/campaignchaincore/js/campaignchain/campaignchain_tracking.js.

 Next, replace [CAMPAIGNCHAIN CHANNEL TRACKING ID] with the ID generated by CampaignChain for your
 channel.

 <script type="text/javascript" src="[CAMPAIGNCHAIN INSTALLATION]/bundles/campaignchaincore/js/campaignchain/campaignchain_tracking.js"></script>
 <script type="text/javascript">
    var campaignchainChannel = '[CAMPAIGNCHAIN CHANNEL TRACKING ID]';
 </script>
 */

/*
 Includes JQuery if not yet available.
 */
if(typeof jQuery == 'undefined'){
    document.write('<script type="text/javascript" src="http://ajax.aspnetcdn.com/ajax/jQuery/jquery-1.11.1.min.js"></'+'script>');
}

/*
Include JQuery Cookies library.
 */
document.write('<script type="text/javascript" src="http://cdnjs.cloudflare.com/ajax/libs/jquery-cookie/1.4.1/jquery.cookie.min.js"></'+'script>');

/*
 *  Initiate the tracking functionality.
 */
jQuery(document).ready(function() {
    var campaignchain = new CampaignChain();

    // Disable clicks if dev-stay mode and Tracking ID exists in URL.
    if(campaignchain.mode == 'dev-stay' &&
        campaignchain.getTrackingId()){
        jQuery('a').on('click', function(event) {
            event.preventDefault();
        });
    }

    // Has a link been clicked?
    jQuery('a').click(function(){
        campaignchain.sendUrlReport(jQuery(this).attr("href"));
        return false;
    })
    // TODO: Has a form been submitted?
});

/**
 *  Define the CampaignChain class.
 *
 * @constructor
 */
function CampaignChain(){
    /**
     * The name of the CTA Tracking ID, e.g. 'campaignchain-id'.
     *
     * @type {string}
     */
    this.idName = 'campaignchain-id';

    /**
     * Value of the Tracking ID.
     *
     * @type {string|bool}
     */
    this.idValue = null;

    /**
     * The Channel Tracking ID.
     *
     * @type {string}
     */
    this.channel = window.campaignchainChannel;

    /**
     * The URL of the source (i.e. the current) page.
     *
     * @type {string}
     */
    this.source = window.location.href;

    /**
     * Development or production mode.
     *
     * Possible values:
     * - 'prod': Production mode
     * - 'dev': Development mode
     * - 'dev-stay': Click will not be executed and you will stay on the Web page.
     *
     * @type {string} prod|dev|dev-stay
     */
    this.mode = 'dev';
}

/**
 * Sends a CTA report to CampaignChain.
 *
 * @param target
 */
CampaignChain.prototype.sendUrlReport = function(target)
{
    this.target = target;

    // Check if the CampaignChain Tracking ID exists.
    if(this.getTrackingId()){
        // Use Symfony dev environment if dev mode for API URL.
        if(this.mode == 'dev' || this.mode == 'dev-stay'){
            var ajaxUrlMode = 'app_dev.php/';
        } else {
            var ajaxUrlMode = '';
        }
        // Compose the API URL.
        var ajaxUrl = jQuery('script[src$="campaignchain_tracking.js"]').
            attr('src').replace(
                'bundles/campaignchaincore/js/campaignchain/campaignchain_tracking.js',
                ajaxUrlMode + 'api/v1/report/cta/new/' + this.channel);

        if(this.mode == 'dev' || this.mode == 'dev-stay'){
            console.log('API URL: ' + ajaxUrl);
        }

        // Pass the tracking data to the CampaignChain API.
        jQuery.ajax({
            url: ajaxUrl,
            data: { source: this.source, target: this.target },
            dataType: 'jsonp',
            cache: false,
            context: this,

            success: function(data, status) {
                /*
                 If an external target URL, then append the Tracking ID if it is not
                 already appended.
                 */
                this.addTrackingIdToTarget();

                if(this.mode != 'dev-stay'){
                    window.location.href = this.target;
                } else {
                    console.log('AJAX success: ' + status);
                    console.log('Would redirect to: ' + this.target);
                }
            },
            error: function (xhr, ajaxOptions, thrownError) {
                if(this.mode == 'dev-stay'){
                    console.log(
                        'AJAX error: URL: ' + ajaxUrl + ', status: ' + xhr.status +
                        ', message: ' +thrownError
                    );
                }
            }
        });
    }
}

/**
 * Checks whether Tracking ID exists as parameter in URL of this page.
 *
 * @returns {boolean}
 */
CampaignChain.prototype.getTrackingId = function()
{
    var logMsg = 'CTA Tracking ID with name "' + this.idName + '"';

    // If not in URL, check if it's in a Cookie.
    if(this.source.toLowerCase().indexOf(this.idName) < 0){
        logMsg =  logMsg + 'is NOT in URL';

        this.idValue = this.getCookie();

        if(this.idValue){
            logMsg = logMsg + ', but is in Cookie.';
        } else {
            logMsg = logMsg + ' and is NOT in Cookie.';
        }
    } else {
        this.idValue = decodeURIComponent((new RegExp('[?|&]' + this.idName + '=' + '([^&;]+?)(&|#|;|$)').exec(this.source)||[,""])[1].replace(/\+/g, '%20'))||null;
        logMsg = logMsg + ' is in URL.';
    }

    if(this.mode == 'dev' || this.mode == 'dev-stay'){
        console.log(logMsg);
        console.log('Tracking ID value: ' + this.idValue);
    }

    return this.idValue;
}

/**
 * Appends the Tracking ID to the target URL.
 *
 * @returns {CampaignChain.target}
 */
CampaignChain.prototype.addTrackingIdToTarget = function()
{
    // If Tracking ID is already in URL, then return as is.
    if(this.target.toLowerCase().indexOf(this.idName) >= 0){
        return this.target;
    }

    // No Tracking ID yet, so add it.
    if (this.target.indexOf(this.idName + "=") >= 0)
    {
        var prefix = this.target.substring(0, this.target.indexOf(this.idName));
        var suffix = this.target.substring(this.target.indexOf(this.idName));
        suffix = suffix.substring(suffix.indexOf("=") + 1);
        suffix = (suffix.indexOf("&") >= 0) ? suffix.substring(suffix.indexOf("&")) : "";
        this.target = prefix + this.idName + "=" + this.idValue + suffix;
    }
    else
    {
        if (this.target.indexOf("?") < 0)
            this.target += "?" + this.idName + "=" + this.idValue;
        else
            this.target += "&" + this.idName + "=" + this.idValue;
    }
    return this.target;
}

/**
 * Returns the Tracking ID value from the Cookie.
 *
 * @returns {string}
 */
CampaignChain.prototype.getCookie = function() {
    return jQuery.cookie(this.idName);
}